#include <Max3421e.h>
#include <Usb.h>
#include <AndroidAccessory.h>
#include <Servo.h> 
#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>

// The shield uses the I2C SCL and SDA pins. On classic Arduinos
// this is Analog 4 and 5 so you can't use those for analogRead() anymore
// However, you can connect other I2C sensors to the I2C bus and share
// the I2C bus.
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

// These #defines make it easy to set the backlight color
#define RED 0x1
#define YELLOW 0x3
#define GREEN 0x2
#define TEAL 0x6
#define BLUE 0x4
#define VIOLET 0x5
#define WHITE 0x7



// Data types
#define DISTANCE_READING   0x1

#define SERVO_ID_1         0x1
#define SERVO_ID_1_PIN     2

#define PING_SENSOR_PIN    4

#define COMMAND_TEXT       0xF 
#define TARGET_DEFAULT     0xF

Servo servo;

// AndroidAccessory acc("Manufacturer", "Model", "Description", "Version", "URI", "Serial");
AndroidAccessory acc("Tomahawk",
		     "RobotBrain",
		     "Robot Brain",
		     "1",
		     "http://www.tomahawk.ca",
		     "00123456");

byte rxmsg[255];
byte txmsg[255];

int posInDegrees = 80;
char buffer[ 256 ];

int line = 0;

void setup() 
{
  Serial.begin(19200);
  servo.attach(SERVO_ID_1_PIN);
  acc.powerOn();
  
    // set up the LCD's number of rows and columns: 
  lcd.begin(16, 2);
  
  // Print a message to the LCD. We track how long it takes since
  // this library has been optimized a bit and we're proud of it :)
  int time = millis();
  lcd.print("Hello, world!");
  time = millis() - time;
  Serial.print("Took "); Serial.print(time); Serial.println(" ms");
  lcd.setBacklight(BLUE);
}


void loop() 
{
   if (acc.isConnected())
   {
      int len = acc.read(rxmsg, sizeof(rxmsg), 1); 
      if (len > 0) 
      { 
         if (rxmsg[0] == COMMAND_TEXT) 
         { 
            if (rxmsg[1] == TARGET_DEFAULT)
            { 
               // get the textLength from the checksum byte 
              
               byte textLength = rxmsg[2]; 
               int textEndIndex = 3 + textLength; 
               rxmsg[textEndIndex] = 0;
               
               Serial.println( (char*)rxmsg ); 

               delay(250);              
            } 
         }          
      }
      
      long duration, inches, cm;
      posInDegrees = map( posInDegrees, 0, 180, 0, 180 );

      servo.write( posInDegrees );
     
      do{
         // give the servo time to reach its position
         delay(50);
         
         // The PING))) is triggered by a HIGH pulse of 2 or more microseconds.
         // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:
         pinMode(PING_SENSOR_PIN, OUTPUT);
         digitalWrite(PING_SENSOR_PIN, LOW);
         delayMicroseconds(2);
         digitalWrite(PING_SENSOR_PIN, HIGH);
         delayMicroseconds(5);
         digitalWrite(PING_SENSOR_PIN, LOW);   

         // The same pin is used to read the signal from the PING))): a HIGH
         // pulse whose duration is the time (in microseconds) from the sending
         // of the ping to the reception of its echo off of an object.
         pinMode(PING_SENSOR_PIN, INPUT);
         duration = pulseIn(PING_SENSOR_PIN, HIGH);
         
         if( duration == 0 )
            Serial.println( "Taking a new reading..." );
         else
         {
            inches = microsecondsToInches(duration);
            cm = microsecondsToCentimeters(duration);
         }
            
      }while( duration==0 );
      
      Serial.print( "At " );
      Serial.print( posInDegrees );
      Serial.print( " degrees the distance is " );
      Serial.print(inches);
      Serial.print("in, ");
      Serial.print(cm);
      Serial.print("cm");
      Serial.println();
   
      txmsg[0] = DISTANCE_READING;
      txmsg[1] = (byte)(posInDegrees >> 8); 
      txmsg[2] = (byte)(posInDegrees); 
   
      txmsg[3] = (byte)(inches >> 16);
      txmsg[4] = (byte)(inches >> 8);
      txmsg[5] = (byte)(inches);
      
      acc.write(txmsg, 6);
      delay(250);
   
      posInDegrees += 10;
      if( posInDegrees >= 180 ) 
         posInDegrees = 0;
         
      sprintf( buffer, "%s%d deg = %d\"  ", ((posInDegrees<100)?"  ":" "), posInDegrees, inches );
      LCDprint( buffer );
   }
}

void LCDprint( char* msg )
{  
   uint8_t buttons = lcd.readButtons();
   if (buttons) 
   {
      lcd.clear();
      lcd.setCursor(0,0);
      if (buttons & BUTTON_UP) {
         lcd.print("UP ");
         lcd.setBacklight(RED);
      }
      if (buttons & BUTTON_DOWN) {
         lcd.print("DOWN ");
         lcd.setBacklight(YELLOW);
      }
      if (buttons & BUTTON_LEFT) {
         lcd.print("LEFT ");
         lcd.setBacklight(GREEN);
      }
      if (buttons & BUTTON_RIGHT) {
         lcd.print("RIGHT ");
         lcd.setBacklight(TEAL);
      }
      if (buttons & BUTTON_SELECT) {
         lcd.print("SELECT ");
         lcd.setBacklight(VIOLET);
      }
   }
   
   lcd.setCursor( 0, line++ );   //  column 0, line 1
   lcd.print( msg );
   
   if( line == 2 )
      line=0;
}

long microsecondsToInches(long microseconds)
{
  // According to Parallax's datasheet for the PING))), there are
  // 73.746 microseconds per inch (i.e. sound travels at 1130 feet per
  // second).  This gives the distance travelled by the ping, outbound
  // and return, so we divide by 2 to get the distance of the obstacle.
  // See: http://www.parallax.com/dl/docs/prod/acc/28015-PING-v1.3.pdf
  return microseconds / 74 / 2;
}

long microsecondsToCentimeters(long microseconds)
{
  // The speed of sound is 340 m/s or 29 microseconds per centimeter.
  return microseconds / 29 / 2;
}
